@@ -49,7 +49,8 @@
 		   :if-does-not-exist nil))
     (format t "Sorry, there is no compiled version of this file.")))
 
-(let ((*default-pathname-defaults* (pathname *NESL-PATH*)))
+(let ((*default-pathname-defaults* (pathname *nesl-path*)))
+  (declare (special *default-pathname-defaults*))
   (load "make")
   (load-bin-file "packages")
   (load-bin-file "funspec")
@@ -63,25 +64,33 @@
   (load-bin-file "vcode-lisp")
   (load-bin-file "defop")
   (load-bin-file "read-eval-print")
-  (load "src/config.lisp")
 
+  ;; CNESL SYNTAX STUFF
+  (load-bin-file "cgtoke")
+  (load-bin-file "cgol-cl")
+  (load-bin-file "cnesl")
+  (speaknesl)
+
+  (load "src/config.lisp")
+      
   (let ((*redefine-default* t))
     (declare (special *redefine-default*))
     (load-nesl "src/types.nesl" :print nil)
     (load-nesl "src/scalar-ops.nesl" :print nil)
     (load-nesl "src/vector-ops.nesl" :print nil)
     (load-nesl "src/nest-ops.nesl" :print nil)
-    (load-nesl "src/other-ops.nesl" :print nil)
-    (load-nesl "src/io.nesl" :print nil))
+    (load-nesl "src/other-ops.cnesl" :print nil)
+    (load-nesl "src/io.nesl" :print nil)
+    (load-nesl "src/ltoc.nesl" :print nil))
 
   (format t "~%")
   (init-machine-configuration)
 
-  (format t "~%; -> You can switch to Lisp by typing (lisp) and back ~
-                to Nesl by typing (nesl).~%~
-                ; -> Use (nesl) when an error aborts you out of the ~
-                interpreter.~%~%~
-                ; -> TYPE (HELP) FOR A LIST OF THE TOP LEVEL COMMANDS.~%~%")
+  (format t "~%~%; -> You can switch to Lisp by typing lisp();~%~
+                 ; -> and back to Nesl by typing (nesl).~%~
+                 ; -> Use (nesl) when an error aborts you out of the ~
+                 interpreter.~%~
+                 ; -> Type help(); for a list of the top level commands.~%~%")
   )
 
 (nesl)
