@@ -9,22 +9,28 @@
   (error "The NESL/VCODE interface has not been implemented ~
           for the Lisp you are using."))
 
-(let* ((src-dir (concatenate 'string *NESL-PATH* "src/"))
-       (obj-dir (concatenate 'string *NESL-PATH* "bin/")))
+(let* ((root-directory *nesl-path*)
+       #+(or allegro excl) (obj-tailer ".fasl")
+       #+lucid             (obj-tailer ".sbin")
+       #+cmu               (obj-tailer ".f")
+       (src-dir (concatenate 'string root-directory "/src/"))
+       (obj-dir (concatenate 'string root-directory "/bin/")))
   (flet ((comp-file (name &key needs)
 	   (let ((src-file (concatenate 'string src-dir name ".lisp"))
-		 (obj-file (concatenate 'string obj-dir name *obj-tailer*)))
+		 (obj-file (concatenate 'string obj-dir name obj-tailer)))
 	     (when (not (probe-file src-file))
 		   (error "No source file ~a." src-file))
 	     (when (or (not (probe-file obj-file))
 		       (< (file-write-date obj-file)
 			  (file-write-date src-file)))
 	       (when needs (load needs))
-	       (compile-file src-file :output-file obj-file)))))
+	       (compile-file src-file :output-file obj-file))))
+	 (load-file (name)
+	    (load (concatenate 'string obj-dir name obj-tailer))))
     (comp-file "packages")
-    (load (concatenate 'string src-dir "packages.lisp"))
+    (load-file "packages")
     (comp-file "funspec")
-    (load (concatenate 'string src-dir "funspec.lisp"))
+    (load-file "funspec")
     (comp-file "free")
     (comp-file "type")
     (comp-file "type-check")
@@ -35,7 +41,14 @@
     (comp-file "vcode-lisp")
     (comp-file "defop")
     (comp-file "read-eval-print")
+    (comp-file "cgtoke")
+    (comp-file "cgol-cl")
+    (load-file "cgtoke")
+    (load-file "cgol-cl")
+    (comp-file "cnesl")
 
     (when (probe-file "vcode/Makefile")
        (format t "~%; Compiling CVL and VCODE~%~%")
-       (execute-shell-line "cd vcode; make"))))
+       (execute-shell-line (format nil "cd ~avcode; make" *nesl-path*)))
+    ))
+
