@@ -87,36 +87,36 @@ Returns a string if defined or nil if undefined."
 
 (defun nesl::configuration ()
   (let ((machine-name (config-machine-name *current-config*)))
-    (format t "Machine Name:      ~s~%~
-               Interpreter File:  ~s~%~
-               Memory Size:        ~s~%~
-               Argument Checking:  ~s~%"
+    (format t "~%Machine Name:      ~s~
+               ~%Interpreter File:  ~s~
+               ~%Memory Size:       ~s~
+               ~%Argument Checking: ~s"
 	    machine-name
 	    (config-interp-file *current-config*)
 	    (config-memsize *current-config*)
 	    (config-checkargs *current-config*))
     (when (not (equal machine-name "local"))
-	  (format t "rsh Command:       ~s~%"
+	  (format t "~%rsh Command:       ~s"
 		  (config-rsh-command *current-config*))
-	  (format t "Remote User Name:  ~s~%"
+	  (format t "~%Remote User Name:  ~s"
 		  (config-user-name *current-config*)))
     (when (not (config-afs *current-config*))
-	  (format t "No access to AFS, will be using rcp for copying.~%"))))
+	  (format t "~%No access to AFS, will be using rcp for copying."))))
 
 (defun nesl::list-machines ()
-  (format t "The current machine configurations are:~%")
-  (mapcar #'(lambda (a) (format t "~a~%" (car a))) (reverse *config-list*))
-  (format t "To use one type: (use-machine NAME).~%")
+  (format t "~%The current machine configurations are:")
+  (mapcar #'(lambda (a) (format t "~%~a" (car a))) (reverse *config-list*))
+  (format t "~%To use one type: (use-machine NAME).")
   nil)
 
-(defun nesl::listmachines () (nesl::list-machines))
+(defun nesl::list_machines () (nesl::list-machines))
 
 (defun nesl::set-memsize (size)
   (when (not (integerp size))
     (nesl-error "The size argument to SET-MEMSIZE must be an integer."))
   (setf (config-memsize *current-config*) size))
 
-(defun nesl::setmemsize (size) (nesl::set-memsize size))
+(defun nesl::set_memory_size (size) (nesl::set-memsize size))
 
 (defun nesl::set-interpreter-file (filename)
   (setf (config-interp-file *current-config*) filename))
@@ -135,12 +135,12 @@ Returns a string if defined or nil if undefined."
 		(setq *current-config* config)
 		(nesl::configuration))
 	       (t
-		(format t "Machine ~a is not a valid configuration.~%" 
+		(format t "~%Machine ~a is not a valid configuration." 
 			',config-name)
 		(nesl::list-machines))))
     `(nesl::list-machines)))
 
-(defmacro nesl::usemachine (&optional config-name)
+(defmacro nesl::use_machine (&optional config-name)
   `(nesl::use-machine ,config-name))
 
 (defun nesl::cm-finger ()
@@ -155,7 +155,7 @@ Returns a string if defined or nil if undefined."
 
 (defun init-machine-configuration ()
   (let ((config (or (getenv "NESL_CONFIG") "local")))
-    (format t "; Using machine configuration ~s.~%" config)
+    (format t "~%; Using machine configuration ~s." config)
     (setq *current-config* 
 	  (cdr (assoc (let ((*package* (find-package 'nesl)))
 			(declare (special *package*))
@@ -199,7 +199,7 @@ Returns a string if defined or nil if undefined."
     (cond 
      ;; FOR RUNNING VCODE LOCALLY
      ((equal machine-name "local")
-      (when *verbose* (format t "Starting VCODE process locally...~%"))
+      (when *verbose* (format t "~%Starting VCODE process locally..."))
       (run-shell-line
        (format nil "~a~{ ~a~} ~a > ~a"
 	       interp-file flags vcodefile outfile)))
@@ -207,7 +207,7 @@ Returns a string if defined or nil if undefined."
      ;; RUNNING VCODE REMOTELY ON MACHINE WITHOUT AFS ACCESS
      ((not afs)
       (when *verbose* 
-	(format t "Copying control file to ~a...~%"
+	(format t "~%Copying control file to ~a..."
 		(string-upcase machine-name)))
       (run-shell-line
        (format nil "~a -l ~a ~a ~
@@ -220,7 +220,7 @@ Returns a string if defined or nil if undefined."
      ;; RUNNING VCODE REMOTELY ON MACHINE WITH AFS ACCESS
      (t
       (when *verbose* 
-	(format t "Starting VCODE process on remote machine ~a...~%"
+	(format t "~%Starting VCODE process on remote machine ~a..."
 		(string-upcase machine-name)))
       (run-shell-line
        (format nil "~a -l ~a ~a ~
@@ -233,12 +233,11 @@ Returns a string if defined or nil if undefined."
   (let ((list (read-from-string line)))
     (when (not (listp list))
       (nesl-error "Error while running VCODE."))
-    (substitute nil 'f list)))
+    ;; (substitute nil 'f list)
+    list))
 
 (defun parse-segdes-line (line)
-  (substitute nil 'f (read-from-string 
-		      (substitute #\( #\[
-				  (substitute #\) #\] line)))))
+  (read-from-string (substitute #\( #\[ (substitute #\) #\] line))))
 
 (defun convert-anything (thing)
   (cond ((typep thing 'integer) thing)
@@ -273,14 +272,14 @@ by run-vcode."
 	 (codefile (merge-pathnames *codefile*))
 	 (outfile (merge-pathnames *outfile*)))
     (declare (special *default-pathname-defaults*))
-    (when *verbose* (format t "Writing VCODE file....~%"))
+    (when *verbose* (format t "~%Writing VCODE file...."))
     (when (probe-file codefile) (delete-file codefile))
     (write-func topsym '() definitions codefile)
     (when (probe-file outfile) (delete-file outfile))
     (execute-vcode-file *codefile* *outfile*)
     (when (not (probe-file outfile))
       (nesl-error "Error while executing VCODE."))
-    (when *verbose* (format t "Reading VCODE output....~%"))
+    (when *verbose* (format t "~%Reading VCODE output...."))
     (with-open-stream (s (open outfile :direction :input))
       (let ((result nil))
 	(dotimes (i (config-munch-lines *current-config*)) 
@@ -291,11 +290,12 @@ by run-vcode."
 		    (nesl-error "Error while executing VCODE."))
 		  (push
 		   (cond
-		    ((member type '(int bool nesl::float char)
+		    ((member type '(int bool nesl::float char nesl::stream)
 			     :test #'equal)
 		     (parse-line line))
 		    ((eql type 'nesl::segdes)
 		     (parse-segdes-line line))
-		    (t (error "Can't handle the result type ~s yet." type)))
+		    (t (nesl-error "Can't handle the result type ~s." 
+				   type)))
 		   result)))
 	(nreverse result)))))
