@@ -124,7 +124,7 @@
     (when (not fundef)
       (nesl-error "Function ~a is undefined (internal error)." fname))
     (let ((full-type (get-instantiated-function-type 
-		      fname arg-type (fundef-type fundef) definitions)))
+		      fundef arg-type definitions)))
       (cons (car full-type) 
 	    (get-call-code fundef arg-type definitions)))))
 
