@@ -46,20 +46,41 @@
 (defun is-unbound-variable? (v definitions)
   (and (var-p v) (variable-type? (var-pointer v) definitions)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; SETTING MUTABLE TYPE VARIABLES
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun check-inclusion-list (var typelist definitions)
+  (if typelist
+      (or (check-inclusion var (car typelist) definitions)
+	  (check-inclusion-list var (cdr typelist) definitions))
+    nil))
+
+(defun check-inclusion (var type definitions)
+  (cond ((is-bound-variable? type definitions)
+	 (check-inclusion var (var-pointer type) definitions))
+	((is-variable? type)
+	 (eq var type))
+	((atom type) nil)
+	(t (check-inclusion-list var (cdr type) definitions))))
+
 (defun set-variable (val newval definitions)
   ;;(format t "val: ~a~%newval: ~a~%~%" val newval)
   (let ((valtype (var-pointer val))
 	(newvaltype (if (is-variable? newval)
 			(var-pointer newval)
 		      newval)))
-    (if (type-< newvaltype valtype definitions)
-	(progn (setf (var-pointer val) newval) nil)
-      (if (type-< valtype newvaltype definitions)
-	  (progn
-	    (setf (var-pointer val) newval)
-	    (setf (var-pointer newval) valtype)
-	    nil)
-	:error1))))
+    (or (check-inclusion val newval definitions)
+	(if (type-< newvaltype valtype definitions)
+	    (progn
+	      (setf (var-pointer val) newval)
+	      nil)
+	  (if (type-< valtype newvaltype definitions)
+	      (progn
+		(setf (var-pointer val) newval)
+		(setf (var-pointer newval) valtype)
+		nil)
+	    :error1)))))
 
 ;;;;;;;;;;;;;;;;;;;;;
 ;;; STRIPPING OFF THE MUTABLE TYPE VARIABLES FROM A TYPE EXPRESSION
@@ -67,7 +88,21 @@
 
 ;;; Used after the completion of type checking.
 
-(defparameter *typevarlist* '(nesl::alpha nesl::beta nesl::gamma nesl::delta))
+;;(defparameter *typevarlist* 
+;;  '(nesl::alpha nesl::beta nesl::gamma nesl::delta
+;;    nesl::epsilon nesl::rho nesl::nu nesl::omega))
+
+(defparameter *typevarlist* 
+  '(nesl::a nesl::b nesl::c nesl::d nesl::e))
+
+(defun typevarnames (n)
+  (if (> n (length *typevarlist*))
+      (do* ((i 0 (+ i 1))
+	   (name (intern (format nil "A~d" i) 'nesl) 
+		 (intern (format nil "A~d" i) 'nesl))
+	   (l (list name) (cons name l)))
+	  ((= i n) (reverse l)))
+    (subseq *typevarlist* 0 n)))
 
 (defun clean-type-list (type definitions)
   (if type
@@ -90,7 +125,7 @@
 (defun clean-type (type definitions)
   (let* ((ctype (clean-type-r type definitions))
 	 (class (mapcar #'var-pointer (cdr ctype)))
-	 (new-vars (subseq *typevarlist* 0 (length (cdr ctype)))))
+	 (new-vars (typevarnames (length (cdr ctype)))))
     (do* ((ftype (car ctype) (subst (car nvars) (car vars) ftype))
 	  (vars (cdr ctype) (cdr vars))
 	  (nvars new-vars (cdr nvars)))
@@ -98,11 +133,11 @@
 
 (defun print-type (type definitions)
   (let ((clean-type (clean-type type definitions)))
-    (cons (pretty-type (car clean-type)) (cdr clean-type))))
+    (pretty-type-full clean-type)))
 
 (defun print-type-list (type definitions)
   (let ((clean-type (clean-type type definitions)))
-    (cons (pretty-type-list (car clean-type)) (cdr clean-type))))
+    (pretty-type-list-full clean-type)))
 
 ;;;;;;;;;;;;;;;;;;;;;
 ;;; UNIFICATION
@@ -171,8 +206,8 @@
 	       (instantiate-fun-type (fundef-type fundef)))))
 	(nesl-error "For function call ~a,~%~
                inferred argument types don't match function specification.~%  ~
-               Argument types: ~{~a ~}~%  ~
-               Function types: ~{~a ~}~%"
+               Argument types: ~a~%  ~
+               Function types: ~a~%"
 		    expression 
 		    (print-type-list arg-types definitions)
 		    (print-type-list (cdr function-types) definitions))))
@@ -184,14 +219,15 @@
 
 ;; These functions are used by trans.lisp in the final generation of code.
 
-(defun get-instantiated-function-type (funname arg-types funspec definitions) 
-  (let* ((function-types (instantiate-fun-type funspec)))
+(defun get-instantiated-function-type (fundef arg-types definitions) 
+  (let* ((function-types (instantiate-fun-type (fundef-type fundef))))
     (when (unify-list arg-types (cdr function-types) definitions)
-	  (nesl-error "For function call ~a,~%~
+	  (nesl-error "INTERNAL ERROR in function call ~a,~%~
                inferred argument types don't match function specification.~%  ~
                Argument types: ~a~%  ~
                Function spec:  ~a~%"
-		      funname (print-type-list arg-types definitions)
+		      (car (fundef-names fundef))
+		      (print-type-list arg-types definitions)
 		      (print-type-list (cdr function-types) definitions)))
     (car (clean-type-r function-types definitions))))
 
@@ -212,7 +248,7 @@
     (nesl-error "The symbol ~a is a constant that cannot be redefined."
 		name))
   (when (not (symbolp name))
-    (nesl-error "The value ~a is not a valid variable name." name)))
+    (nesl-error "The value ~s is not a valid variable name." name)))
 
 (defun bind-typecheck-list (list environment)
   (if list
@@ -358,20 +394,23 @@
 ;;;;;;;;;;;;;;;;;;;;;;;
 
 (defun typecheck-op (names types code definitions)
-  (mapcar #'(lambda (a) (check-valid-varname a definitions))
-	  (cdr names))
-  (let* ((nvars (if types
-		    (instantiate-fun-type types)
-		  (mapcar #'(lambda (a) (declare (ignore a))
-			      (make-variable 'nesl::any))
-			  names)))
+  (let* ((nvars (bind-typecheck-list (cdr names) definitions))
+	 (source-types (cdr nvars))
+	 (full-type (cons (make-variable 'nesl::any) source-types))
+	 (type-check (when (and types (unify-list full-type 
+						  (instantiate-fun-type types)
+						  definitions))
+		       (nesl-error "Supplied type does not match interface.")))
+	 (var-bindings (car nvars))		
 	 (result-type 
-	  (typecheck-exp code (list (mapcar #'cons (cdr names) (cdr nvars))
-				    definitions (cons (car names) nvars)))))
-    (when (unify result-type (car nvars) definitions)
-      (nesl-error "Inferred result type~%  ~a~%~
-                   does not match user supplied result type~%  ~a."
+	  (typecheck-exp code (list var-bindings
+				    definitions
+				    (cons (car names) full-type)))))
+    (when (unify result-type (car full-type) definitions)
+      (nesl-error "Inferred result type~% ~a~%~
+                   does not match either a user supplied result type,~%~
+                   or the result type in a recursive call:~% ~a."
 		  (print-type result-type definitions)
-		  (print-type (car nvars) definitions)))
-    (clean-type nvars definitions)))
+		  (print-type (car full-type) definitions)))
+    (clean-type full-type definitions)))
 
