@@ -66,6 +66,8 @@
 
 (defstruct (nested-sequence (:print-function print-nested-sequence)) data type)
 
+(defstruct (nesl-struct (:print-function print-nesl-struct)) data type)
+
 (defun print-vcode-vector (self stream depth)
   (declare (ignore depth))
   (if (and (= (length (vcode-vector-data self)) 1)
@@ -85,18 +87,37 @@
     (cond ((eql type 'char)
 	   (format stream "~s" (coerce data 'string)))
 	  ((= (length data) 0)
-	  ;; (format stream "#~a.~s" vcode-sequence-letter
-	  ;;	   (nested-sequence-type self))
-	   (format stream "#~a()" vcode-sequence-letter))
+	   (if *cnesl-syntax*
+	       (format stream "[]" vcode-sequence-letter)
+	     (format stream "#~a()" vcode-sequence-letter)))
 	  ((<= (length data) *max-print-length*)
-	   (format stream "#~a~s" vcode-sequence-letter data))
-	  (t (format stream "#~a(~{~s ~}...)" vcode-sequence-letter 
-		     (subseq data 0 *max-print-length*))))))
+	   (if *cnesl-syntax*
+	       (format stream "[~s~{,~s~}]" (car data) (cdr data))
+	     (format stream "#~a~s" vcode-sequence-letter data)))
+	  (t 
+	   (let ((data (subseq data 0 *max-print-length*)))
+	     (if *cnesl-syntax*
+		 (format stream "[~s~{,~s~},...]" (car data) (cdr data))
+	       (format stream "#~a(~{~s ~}...)" vcode-sequence-letter)))))))
+
+(defun print-nesl-struct (self stream depth)
+  (declare (ignore depth))
+  (let ((data (nesl-struct-data self))
+	(type (nesl-struct-type self)))
+    (cond ((not *cnesl-syntax*)
+	   (format stream "(~s~{ ~s~})" type data))
+	  ((eql type 'nesl::pair)
+	   (format stream "(~s,~s)" (first data) (second data)))
+	  (t 
+	   (format stream "~s(~s~{,~s~})" type (car data) (cdr data))))))
+
+(defun nesl-floatp (val)
+  (or (floatp val) (eql val 'nesl::infinity)))
 
 (defparameter *typecheck-map*
-  '((int . integerp) (bool . booleanp) (nesl::float . floatp) 
-    (char . characterp) (nesl::stream . integerp)
-    (nesl::segdes . integerp)))
+  '((int . integerp) (bool . booleanp) (nesl::float . nesl-floatp) 
+    (char . characterp) (nesl::segdes . integerp)
+    (nesl::stream . integerp)))
 
 (defun make-vcode-vector-safely (type data)
   (let* ((itype (or type 
@@ -263,7 +284,9 @@
 	     (list 'nesl::vector 'nesl::segdes type))))
 	((and (listp data) (> (length data) 1))
 	 (multiple-value-bind (result type) (nest-constant-list (cdr data))
-	   (values (mapcar #'(lambda (a) (cons (car data) a)) result)
+	   (values (mapcar #'(lambda (a) 
+			       (make-nesl-struct :type (car data) :data a))
+			   result)
 		   (cons (car data) type))))
 	((and (listp data) (= (length data) 1))
 	 (nesl-error "NESL currently cannot print empty records."))
