@@ -29,6 +29,7 @@
 
 #include <stdio.h>
 #include <strings.h>
+#include <errno.h>
 
 #include "vcode.h"
 #include "y.tab.h"
@@ -268,6 +269,9 @@ static void main_loop()
 		case FWRITE:
 		    do_fwrite(instruction);
 		    break;
+		case SPAWN:
+		    do_spawn(instruction);
+		    break;
 		case START_TIMER:
 		    tgt_fos(&start_time);
 		    break;
