@@ -43,7 +43,7 @@ typedef struct prog_entry {
     union {			/* misc stuff */
 	int 	branch;		/* prog_entry number to branch to */
 	int	int_const;	/* value of constants */
-	float	float_const;
+	double	float_const;
 	int	bool_const;	
 	int	const_pool_index; /* location of vector constant in pool */
 	struct {		/* args to copy and pop */
