@@ -345,7 +345,7 @@ FILE *stream;
 		double val;
 		assert_mem_size(ext_vub_scratch(len));
 		val = ext_vud(vb_print->vector, i, len, SCRATCH);
-		fprintf(stream, " %#g", val);
+		fprintf(stream, " %.14e", val);
 		break;
 	    }
 	    default:
@@ -407,7 +407,7 @@ FILE *stream;
     /* switch on value_type */
     switch (value_type) {
         case Float:
-	    if (fscanf(stream, "%lf", (double *)buffer) == 1)
+	    if (fscanf(stream, " %lf", (double *)buffer) == 1)
 		return (int*)(1 + (double*)buffer);
 	    break;
 	case Bool:
@@ -425,8 +425,21 @@ FILE *stream;
 		}
 	    }
 	    break;
+	case String:
+	    /* For now, just get a char.  Eventually, we would like '\' 
+	     * to do something useful.
+	     */
+	    c = getc(stream);
+	    if ((c != (int) '\"') && (c != EOF)) {
+		*buffer = c;
+		return ++buffer;
+	    } else {
+		(void) ungetc(c, stream);
+		return NULL;
+	    }
+	    break;
 	case Int:
-	    if (fscanf(stream, "%d", buffer) == 1) 
+	    if (fscanf(stream, " %d", buffer) == 1) 
 		return ++buffer;
 	    break;
 	default:
@@ -438,7 +451,7 @@ FILE *stream;
 
 /* perform the WRITE function; write_value() and write_string() do the work.
  * This puts two values on the stack: 
- *    boolean - T if the read was successful, F otherwise
+ *    boolean - T if the write was successful, F otherwise
  *    error-string - null if no error, otherwise an error message
  */
 void do_write(pe, stream)
@@ -478,11 +491,13 @@ FILE *stream;
     vec_p vector;			/* where to copy input */
     TYPE type = pe->type;
     TYPE read_type;			/* type of data being read: for type
-					 * Segdes, we read Int */
+					 * Segdes -> Int  */
+    TYPE stack_type;			/* type of vector element:
+					 * String -> Int */
     int c;				/* char read in */
     int *new_value_buffer;		/* for reading in data */
 
-    char *error_msg;
+    char error_msg[256];
     int error_flag= 0;
 
     int start_vector;			/* must appear as first char */
@@ -504,6 +519,10 @@ FILE *stream;
 		type_string = "boolean";
 		form_string = "(T F)";
 		break;
+	    case String:
+		type_string = "string";
+		form_string = "\"string\"";
+		break;
 	    case Segdes:
 		type_string = "segment descriptor";
 		form_string = "[1 2 3]";
@@ -516,8 +535,27 @@ FILE *stream;
         fprintf(stderr, "vinterp: type %s vector, in form %s: ", type_string, form_string);
     }
 
-    if (type == Segdes) { start_vector = '['; end_vector = ']'; }
-    else { start_vector = '('; end_vector = ')'; }
+    /* figure out vector delimiters, and types to be read and stored */
+    switch (type) {
+	case Int: case Float: case Bool:
+	    start_vector = '('; end_vector = ')';
+	    stack_type = read_type = type;
+	    break;
+	case String: 
+	    end_vector = start_vector = '"';
+	    read_type = String;
+	    stack_type = Int;
+	    break;
+	case Segdes:
+	    start_vector = '['; end_vector = ']';
+	    read_type = Int;
+	    stack_type = Segdes;
+	    break;
+	default:
+	    fprintf(stderr, "vinterp: internal error: improper type %d in do_read().\n", type);
+	    vinterp_exit(1);
+	    break;
+    }
 
     /* skip whitespace */
     while ((c = getc(stream)) != EOF && isspace(c))
@@ -526,19 +564,14 @@ FILE *stream;
     /* get start of vector */
     if (c != start_vector) {
 	error_flag = 1;
-	if (type == Segdes) 
-	    error_msg = "input error: vector must start with '['";
-	else
-	    error_msg = "input error: vector must start with '('";
+	(void) sprintf(error_msg, "input error: vector must start with %c", start_vector);
 	goto ERROR;
     }
 
-    if (type == Segdes) read_type = Int;
-    else read_type = type;
-
-    /* read values until we're done */
     new_value_buffer = read_array;
     count = 0;
+
+    /* read values until we're done */
     while (1) {
 	/* check to see if room in buffer, reallocate if not */
 	if (new_value_buffer >= read_array + input_size_max) {
@@ -550,7 +583,7 @@ FILE *stream;
 	        vinterp_exit(1);
 	    } else {
 		switch (read_type) {
-		    case Int: case Bool:
+		    case Int: case Bool: case String:
 			new_value_buffer = read_array + count;
 			break;
 		    case Float:
@@ -563,6 +596,7 @@ FILE *stream;
 	    }
 	}
 
+	/* do the input */
 	new_value_buffer = get_value(new_value_buffer, read_type, stream);
 	if (new_value_buffer == NULL)
 	    break;
@@ -576,11 +610,7 @@ FILE *stream;
     /* get end of vector */
     if (c != end_vector) {
 	error_flag = 1;
-	if (type == Segdes)
-	    error_msg = "vinterp: input error: vector must end with ')'";
-	else 
-	    error_msg = "vinterp: input error: vector must end with ']'";
-
+	(void) sprintf(error_msg, "input error: vector must end with %c", end_vector);
 	/* things failed, put out error message and return */
 ERROR:
 	{
@@ -602,10 +632,10 @@ ERROR:
 	}
     }
 
-    stack_push(count, read_type, -1);	/* allocate stack space */
+    stack_push(count, stack_type, -1);	/* allocate stack space */
     vector = se_vb(TOS)->vector;
     for (i=0; i < count; i++) {		/* copy values in, one by one ...  */
-	switch (type) {
+	switch (stack_type) {
 	    case Int:
 	    case Segdes: 	/* create the lengths vector for a segdes */
 		assert_mem_size(rep_vuz_scratch(count));
@@ -667,11 +697,46 @@ ERROR:
 
 extern char *sys_errlist[];
 
-/* file_list : list of open files and their file descriptors;
- * NOFILES is max number open files for process,
+/* file_list: List of open files and their file descriptors.
+ *	      The index of a FILE in file_list will be used by vcode
+ *	      to refer to the file.
+ *	      We initialize file_list so that stdin, stdout, and stderr
+ *	      are already present.
+ * NOFILE is max number open files for a process.
  */
 
-static FILE *file_list[NOFILE];   
+static FILE *file_list[NOFILE] = {stdin, stdout, stderr, };   
+
+/* Check that these were defined correctly in vcode.h */
+/* Removed #error "*_FD defined incorrectly in vcode.h" -- Guy */
+#if (STDIN_FD != 0 || STDOUT_FD != 1 || STDERR_FD !=2)
+#if _STDC_
+*_FD defined incorrectly in vcode.h
+#else
+*_FD defined incorrectly in vcode.h
+#endif
+#endif
+
+/* add a file to file_list.  return index into list, -1 if error */
+static int fd_insert(file)
+FILE *file;
+{
+    int fd;
+
+    for (fd = 0; (fd < NOFILE) && (file_list[fd] != (FILE *)NULL); fd ++)
+	;
+
+    if (fd < NOFILE) {
+	/* everything ok */
+	assert(file_list[fd] == (FILE *)NULL);
+	file_list[fd] = file;
+    } else {
+	/* no room in file table */
+	fd = -1;
+    }
+
+    return fd;
+}
 
 /* open a file.  Take two arguments off the stack: 
  *    filename - string (must convert CVL string to C string)
@@ -960,3 +1025,228 @@ prog_entry_t *pe;
 	do_write(pe, file);
     }
 }
+
+/* get the top of stack and turn it into an fd.
+ * Return an error message if problem, NULL if ok.
+ */
+char *fd_from_stack(pe, pfd)
+prog_entry_t *pe;
+int *pfd;
+{
+    vb_t *fd_vb;				/* file descriptor vector */
+    int fd;					/* int form of fd */
+    char *error_msg = NULL;			/* error msg to return */
+
+    fd_vb = se_vb(TOS);
+
+    /* check type and length */
+    if (check_args && (fd_vb->type != Int) && (fd_vb->len != 1) ) {
+	fprintf(stderr, "vinterp: line %d: %s: top of stack not legal file descriptor.\n",
+	       pe->lineno, pe->vopdes->vopname);
+	vinterp_exit(1);
+    }
+
+    /* get value */
+    assert_mem_size(ext_vuz_scratch(1));
+    fd = (int) ext_vuz(fd_vb->vector, 0, 1, SCRATCH);
+
+    /* check for validity */
+    if ((fd != NULL_STREAM_FD) &&
+	(fd < 0 || fd >= NOFILE || file_list[fd] == NULL)) {
+	/* invalid: write results to stack */
+	error_msg = "illegal VCODE file descriptor";
+    }
+
+    stack_pop(TOS);		/* remove from stack */
+
+    *pfd = fd;
+    return error_msg;
+}
+
+/* do_spawn() : This allows VCODE to communicate with other C processes.
+ * The SPAWN command takes 4 args from the stack:
+ *    execution string - a string that will be passed to execvp
+ *    in_fd -  a file descriptor -  stdin of new process
+ *    out_fd - a file descriptor -  stdout of new process
+ *    err_fd - a file descriptor -  stderr of new process
+ * Any of these arguments may be the NULL_STREAM_FD, in which case,
+ * new streams are created.
+ * The command returns three file descriptors, a boolean status flag 
+ * and an error message.
+ */
+void do_spawn(pe)
+prog_entry_t *pe;
+{
+    vb_t *command_vb;				/* command to run (vector) */
+    char *command;				/* command to run (string) */
+    int command_len;				/* length of command vector */
+    int len, i;					/* temporary */
+
+    int in_fd, out_fd, err_fd;			/* int form of fds */
+
+    int error_flag = 0;				/* has error occurred? */
+    char *error_msg = "";			/* error msg to return */
+    char *err_msg, *out_msg, *in_msg;		/* returns of fd_from_stack */
+
+    int child_in_fd[2], child_out_fd[2], child_err_fd[2];  /* for pipe */
+
+    int isParent;				/* return of fork() */
+
+    /* get the various fd from the stack */
+    err_msg = fd_from_stack(pe, &err_fd);  error_flag |= (err_msg != NULL);
+    out_msg = fd_from_stack(pe, &out_fd);  error_flag |= (out_msg != NULL);
+    in_msg = fd_from_stack(pe, &in_fd);	   error_flag |= (in_msg != NULL);
+
+    if (error_flag) {
+	error_msg = "illegal Vcode file descriptor";
+	goto ERROR;
+    }
+
+    /* now get the command */
+    command_vb = se_vb(TOS);
+    if (check_args && (command_vb->type != Int)) {
+	fprintf(stderr, "vinterp: line %d: SPAWN: first arg not string.\n", pe->lineno);
+	vinterp_exit(1);
+    }
+
+    /* convert command vector to C string */
+    command_len = command_vb->len;
+    command = (char *)malloc((command_len+1) * sizeof(char));
+    if (command == NULL) {
+	fprintf(stderr, "vinterp: line %d: internal error: couldn't allocate command string (len = %d) for SPAWN\n", pe->lineno, command_len);
+	vinterp_exit(1);
+    }
+    assert_mem_size(ext_vuz_scratch(command_len));
+    for (len = 0; len < command_len; len++) {
+	command[len] = (char) ext_vuz(command_vb->vector, len, 
+				       command_len, SCRATCH);
+    }
+    command[len] = '\0';		/* terminate string */
+
+    stack_pop(TOS);			/* remove command name from stack */
+
+    /* Now we can do the spawning */
+    
+    /* For each NULL_STREAM argument, we must set up a pipe
+     * so that the two processes can communicate.
+     */
+    if (in_fd == NULL_STREAM_FD)  pipe(child_in_fd);
+    if (out_fd == NULL_STREAM_FD) pipe(child_out_fd);
+    if (err_fd == NULL_STREAM_FD) pipe(child_err_fd);
+    
+    /* Second, fork off a subprocess */
+    isParent = fork();
+
+    if (isParent) { 	/* parent process will be the interpreter */
+	/* if NULL_STREAM_FD was used, get the corect end of the pipe,
+	 * convert the fd into a FILE *, put it in the file_list,
+	 * and close other end.
+	 */
+	FILE * file = stdin;		/* initialized to non-NULL */
+
+	if (in_fd == NULL_STREAM_FD) {
+	    file = fdopen(child_in_fd[1], "w");  /* should check for error */
+	    in_fd = fd_insert(file);
+	    close(child_in_fd[0]);
+	}
+	if (out_fd == NULL_STREAM_FD) {
+	    file = fdopen(child_out_fd[0], "r");  /* should check for error */
+	    out_fd = fd_insert(file);
+	    close(child_out_fd[1]);
+	}
+	if (err_fd == NULL_STREAM_FD) {
+	    file = fdopen(child_err_fd[0], "r");  /* should check for error */
+	    err_fd = fd_insert(file);
+	    close(child_err_fd[1]);
+	}
+
+	if (in_fd == -1 || out_fd == -1 || err_fd == -1) {
+	    error_msg = "ran out of VCODE descriptors";
+	    error_flag = 1;
+	} else if (file == NULL) {
+	    error_msg = "could not perform fdopen() during spawn";
+	    error_flag = 1;
+	}
+    } else {		/* child will exec the command */
+	int argc;	/* arg count for subprocess */
+	char **argv;
+	int i;
+	int inspace;	/* for command parser */
+
+	/* must break the command string up into pieces */
+	argv = (char **) malloc (sizeof (char *) * command_len);
+	argc = 0;
+	inspace = 1;	/* start on space to catch beginning */
+	for (i = 0; i < command_len; i++) {
+	    if (isspace(command[i])) {
+		command[i] = NULL;
+		inspace = 1;
+	    } else if (inspace) {
+		/* found new word: went from space to non-space */
+		argv[argc++] = command + i;
+		inspace = 0;
+	    }
+	}
+	argv[argc] = NULL;
+
+	/* Set up IO strams for child.  If NULL_STREAM was specified, 
+	 * make correct end of pipe into appropriate stream, and close other
+	 * end.  Otherwise, use supplied stream. 
+	 */
+	if (in_fd == NULL_STREAM_FD) {
+	    dup2(child_in_fd[0], fileno(stdin));
+	    close(child_in_fd[1]);
+	} else 
+	    dup2(fileno(file_list[in_fd]), fileno(stdin));
+
+	if (out_fd == NULL_STREAM_FD) {
+	    dup2(child_out_fd[1], fileno(stdout));
+	    close(child_out_fd[2]);
+	} else 
+	    dup2(fileno(file_list[out_fd]), fileno(stdout));
+
+	if (err_fd == NULL_STREAM_FD) {
+	    dup2(child_err_fd[1], fileno(stderr));
+	    close(child_err_fd[2]);
+	} else 
+	    dup2(fileno(file_list[err_fd]), fileno(stderr));
+
+
+	/* use execvp (search through PATH) to start co-process */
+	execvp(*argv, argv);
+
+	/* reach here only on error */
+	fprintf(stderr, "vinterp: line %d internal error: execvp failed:\n",
+		 pe->lineno);
+	fprintf(stderr, "\t%s\n", sys_errlist[errno]);
+	fprintf(stderr, "\tcommand was %s\n", command);
+
+	vinterp_exit(1);
+    }
+
+ERROR:
+    /* put results on stack */
+    stack_push(1, Int, -1);		/* err_fd */
+    assert_mem_size(rep_vuz_scratch(1));
+    rep_vuz(se_vb(TOS)->vector, 0, err_fd, 1, SCRATCH);
+
+    stack_push(1, Int, -1);		/* out_fd */
+    assert_mem_size(rep_vuz_scratch(1));
+    rep_vuz(se_vb(TOS)->vector, 0, out_fd, 1, SCRATCH);
+
+    stack_push(1, Int, -1);		/* in_fd */
+    assert_mem_size(rep_vuz_scratch(1));
+    rep_vuz(se_vb(TOS)->vector, 0, in_fd, 1, SCRATCH);
+
+    len = strlen(error_msg);		/* error message */
+    stack_push(len, Int, -1);
+    assert_mem_size(rep_vuz_scratch(1));
+    for (i = 0; i < len; i++) {		/* put string on stack */
+	rep_vuz(se_vb(TOS)->vector, i, (int) error_msg[i], len, SCRATCH);
+    }
+
+    stack_push(1, Bool, -1);		/* negation of error_flag */
+    assert_mem_size(rep_vub_scratch(1));
+    rep_vuz(se_vb(TOS)->vector, 0, (cvl_bool) !error_flag, 1, SCRATCH);
+}
+
