@@ -99,7 +99,7 @@ int size_needed;
     }
     compact_mem();          /* try to make more room */
     if (size_needed > scratch_block->size) {
-        fprintf(stderr, "vinterp: ran out of memeory. Rerun with larger -m argument.\n");
+        fprintf(stderr, "vinterp: ran out of memory. Rerun with larger -m argument.\n");
         exit (1);
     }
 }
